@@ -1,48 +1,71 @@
 package util
 
 import (
+	"context"
+	"fmt"
+	"sort"
 	"strings"
-	"time"
 
-	"github.com/bradfitz/slice"
 	"github.com/percona/percona-toolkit/src/go/mongolib/proto"
-	"github.com/percona/pmgo"
 	"github.com/pkg/errors"
-	mgo "gopkg.in/mgo.v2"
-	"gopkg.in/mgo.v2/bson"
+	"go.mongodb.org/mongo-driver/bson/primitive"
+	"go.mongodb.org/mongo-driver/mongo"
+	"go.mongodb.org/mongo-driver/mongo/options"
 )
 
 var (
 	CANNOT_GET_QUERY_ERROR = errors.New("cannot get query field from the profile document (it is not a map)")
 )
 
-func GetReplicasetMembers(dialer pmgo.Dialer, di *pmgo.DialInfo) ([]proto.Members, error) {
-	hostnames, err := GetHostnames(dialer, di)
+func GetReplicasetMembers(ctx context.Context, clientOptions *options.ClientOptions) ([]proto.Members, error) {
+	client, err := mongo.NewClient(clientOptions)
+	if err != nil {
+		return nil, errors.Wrap(err, "cannot get a new client for GetReplicasetMembers")
+	}
+	if err := client.Connect(ctx); err != nil {
+		return nil, errors.Wrap(err, "cannot connect to MongoDB")
+	}
+
+	hostnames, err := GetHostnames(ctx, client)
 	if err != nil {
 		return nil, err
 	}
+	if err := client.Disconnect(ctx); err != nil {
+		return nil, errors.Wrapf(err, "cannot disconnect from %v", clientOptions.Hosts)
+	}
+
 	membersMap := make(map[string]proto.Members)
 	members := []proto.Members{}
 
 	for _, hostname := range hostnames {
-		session, err := dialer.DialWithInfo(getTmpDI(di, hostname))
+		client, err = GetClientForHost(clientOptions, hostname)
 		if err != nil {
-			continue
+			return nil, errors.Wrapf(err, "cannot get a new client to connect to %s", hostname)
+		}
+
+		if err := client.Connect(ctx); err != nil {
+			return nil, errors.Wrapf(err, "cannot connect to %s", hostname)
 		}
-		defer session.Close()
-		session.SetMode(mgo.Monotonic, true)
 
 		cmdOpts := proto.CommandLineOptions{}
-		session.DB("admin").Run(bson.D{{"getCmdLineOpts", 1}, {"recordStats", 1}}, &cmdOpts)
+		// Not always we can get this info. For examples, we cannot get this for hidden hosts so
+		// if there is an error, just ignore it
+		res := client.Database("admin").RunCommand(ctx, primitive.D{{"getCmdLineOpts", 1}, {"recordStats", 1}})
+		if res.Err() == nil {
+			if err := res.Decode(&cmdOpts); err != nil {
+				return nil, errors.Wrapf(err, "cannot decode getCmdLineOpts response for host %s", hostname)
+			}
+		}
 
 		rss := proto.ReplicaSetStatus{}
-		if err = session.Run(bson.M{"replSetGetStatus": 1}, &rss); err != nil {
+		res = client.Database("admin").RunCommand(ctx, primitive.M{"replSetGetStatus": 1})
+		if res.Err() != nil {
 			m := proto.Members{
 				Name: hostname,
 			}
 			m.StateStr = strings.ToUpper(cmdOpts.Parsed.Sharding.ClusterRole)
 
-			if serverStatus, err := GetServerStatus(dialer, di, m.Name); err == nil {
+			if serverStatus, err := GetServerStatus(ctx, client); err == nil {
 				m.ID = serverStatus.Pid
 				m.StorageEngine = serverStatus.StorageEngine
 			}
@@ -50,17 +73,18 @@ func GetReplicasetMembers(dialer pmgo.Dialer, di *pmgo.DialInfo) ([]proto.Member
 			continue // If a host is a mongos we cannot get info but is not a real error
 		}
 
+		if err := res.Decode(&rss); err != nil {
+			return nil, errors.Wrap(err, "cannot decode replSetGetStatus response")
+		}
 		for _, m := range rss.Members {
 			if _, ok := membersMap[m.Name]; ok {
 				continue // already exists
 			}
 			m.Set = rss.Set
-			if serverStatus, err := GetServerStatus(dialer, di, m.Name); err == nil {
+			if serverStatus, err := GetServerStatus(ctx, client); err == nil {
 				m.ID = serverStatus.Pid
 				m.StorageEngine = serverStatus.StorageEngine
-				if cmdOpts.Parsed.Sharding.ClusterRole == "" {
-					m.StateStr = m.StateStr
-				} else {
+				if cmdOpts.Parsed.Sharding.ClusterRole != "" {
 					m.StateStr = cmdOpts.Parsed.Sharding.ClusterRole + "/" + m.StateStr
 				}
 				m.StateStr = strings.ToUpper(m.StateStr)
@@ -68,51 +92,49 @@ func GetReplicasetMembers(dialer pmgo.Dialer, di *pmgo.DialInfo) ([]proto.Member
 			membersMap[m.Name] = m
 		}
 
-		session.Close()
+		client.Disconnect(ctx)
 	}
 
 	for _, member := range membersMap {
 		members = append(members, member)
 	}
 
-	slice.Sort(members, func(i, j int) bool { return members[i].Name < members[j].Name })
+	sort.Slice(members, func(i, j int) bool { return members[i].Name < members[j].Name })
 	return members, nil
 }
 
-func GetHostnames(dialer pmgo.Dialer, di *pmgo.DialInfo) ([]string, error) {
-	hostnames := []string{di.Addrs[0]}
-	di.Direct = true
-	di.Timeout = 2 * time.Second
-
-	session, err := dialer.DialWithInfo(di)
-	if err != nil {
-		return hostnames, err
-	}
-	session.SetMode(mgo.Monotonic, true)
-
+func GetHostnames(ctx context.Context, client *mongo.Client) ([]string, error) {
 	// Probably we are connected to an individual member of a replica set
 	rss := proto.ReplicaSetStatus{}
-	if err := session.Run(bson.M{"replSetGetStatus": 1}, &rss); err == nil {
+	res := client.Database("admin").RunCommand(ctx, primitive.M{"replSetGetStatus": 1})
+	if res.Err() == nil {
+		if err := res.Decode(&rss); err != nil {
+			return nil, errors.Wrap(err, "cannot decode replSetGetStatus response for GetHostnames")
+		}
 		return buildHostsListFromReplStatus(rss), nil
 	}
 
-	defer session.Close()
-
 	// Try getShardMap first. If we are connected to a mongos it will return
 	// all hosts, including config hosts
 	var shardsMap proto.ShardsMap
-	err = session.Run("getShardMap", &shardsMap)
-	if err == nil && len(shardsMap.Map) > 0 {
+	smRes := client.Database("admin").RunCommand(ctx, primitive.M{"getShardMap": 1})
+	if smRes.Err() != nil {
+		return nil, errors.Wrap(smRes.Err(), "cannot getShardMap for GetHostnames")
+	}
+	if err := smRes.Decode(&shardsMap); err != nil {
+		return nil, errors.Wrap(err, "cannot decode getShardMap result for GetHostnames")
+	}
+
+	if len(shardsMap.Map) > 0 {
 		// if the only element getShardMap returns is the list of config servers,
 		// it means we are connected to a replicaSet member and getShardMap is not
 		// the answer we want.
-		_, ok := shardsMap.Map["config"]
-		if ok && len(shardsMap.Map) > 1 {
+		if _, ok := shardsMap.Map["config"]; ok {
 			return buildHostsListFromShardMap(shardsMap), nil
 		}
 	}
 
-	return hostnames, nil
+	return nil, fmt.Errorf("cannot get shards map")
 }
 
 func buildHostsListFromReplStatus(replStatus proto.ReplicaSetStatus) []string {
@@ -138,6 +160,8 @@ func buildHostsListFromReplStatus(replStatus proto.ReplicaSetStatus) []string {
 	for _, member := range replStatus.Members {
 		hostnames = append(hostnames, member.Name)
 	}
+	sort.Strings(hostnames) // to make testing easier
+
 	return hostnames
 }
 
@@ -145,13 +169,13 @@ func buildHostsListFromShardMap(shardsMap proto.ShardsMap) []string {
 	/* Example
 	mongos> db.getSiblingDB('admin').runCommand('getShardMap')
 	{
-	        "map" : {
-	                "config" : "localhost:19001,localhost:19002,localhost:19003",
-	                "localhost:17001" : "r1/localhost:17001,localhost:17002,localhost:17003",
-	                "r1" : "r1/localhost:17001,localhost:17002,localhost:17003",
-	                "r1/localhost:17001,localhost:17002,localhost:17003" : "r1/localhost:17001,localhost:17002,localhost:17003",
-	        },
-	        "ok" : 1
+	  "map" : {
+	    "config" : "localhost:19001,localhost:19002,localhost:19003",
+	    "localhost:17001" : "r1/localhost:17001,localhost:17002,localhost:17003",
+	    "r1" : "r1/localhost:17001,localhost:17002,localhost:17003",
+	    "r1/localhost:17001,localhost:17002,localhost:17003" : "r1/localhost:17001,localhost:17002,localhost:17003",
+	  },
+	  "ok" : 1
 	}
 	*/
 
@@ -181,63 +205,53 @@ func buildHostsListFromShardMap(shardsMap proto.ShardsMap) []string {
 			hostnames = append(hostnames, host)
 		}
 	}
+
+	sort.Strings(hostnames)
 	return hostnames
 }
 
-// This function is like GetHostnames but it uses listShards instead of getShardMap
+// GetShardedHosts is like GetHostnames but it uses listShards instead of getShardMap
 // so it won't include config servers in the returned list
-func GetShardedHosts(dialer pmgo.Dialer, di *pmgo.DialInfo) ([]string, error) {
-	hostnames := []string{di.Addrs[0]}
-	session, err := dialer.DialWithInfo(di)
-	if err != nil {
-		return hostnames, err
-	}
-	defer session.Close()
-
+func GetShardedHosts(ctx context.Context, client *mongo.Client) ([]string, error) {
 	shardsInfo := &proto.ShardsInfo{}
-	err = session.Run("listShards", shardsInfo)
-	if err != nil {
-		return hostnames, errors.Wrap(err, "cannot list shards")
+	res := client.Database("admin").RunCommand(ctx, primitive.M{"listShards": 1})
+	if res.Err() != nil {
+		return nil, errors.Wrap(res.Err(), "cannot list shards")
+	}
+	if err := res.Decode(&shardsInfo); err != nil {
+		return nil, errors.Wrap(err, "cannot decode listShards response")
 	}
 
-	if shardsInfo != nil {
-		for _, shardInfo := range shardsInfo.Shards {
-			m := strings.Split(shardInfo.Host, "/")
-			h := strings.Split(m[1], ",")
-			hostnames = append(hostnames, h[0])
-		}
+	hostnames := []string{}
+	for _, shardInfo := range shardsInfo.Shards {
+		m := strings.Split(shardInfo.Host, "/")
+		h := strings.Split(m[1], ",")
+		hostnames = append(hostnames, h[0])
 	}
 	return hostnames, nil
 }
 
-func getTmpDI(di *pmgo.DialInfo, hostname string) *pmgo.DialInfo {
-	tmpdi := *di
-	tmpdi.Addrs = []string{hostname}
-	tmpdi.Direct = true
-	tmpdi.Timeout = 2 * time.Second
-
-	return &tmpdi
-}
-
-func GetServerStatus(dialer pmgo.Dialer, di *pmgo.DialInfo, hostname string) (proto.ServerStatus, error) {
+// GetServerStatus returns the server status by running serverStatus and recordStats
+func GetServerStatus(ctx context.Context, client *mongo.Client) (proto.ServerStatus, error) {
 	ss := proto.ServerStatus{}
 
-	tmpdi := getTmpDI(di, hostname)
-	session, err := dialer.DialWithInfo(tmpdi)
-	if err != nil {
-		return ss, errors.Wrapf(err, "getReplicasetMembers. cannot connect to %s", hostname)
+	query := primitive.D{
+		{Key: "serverStatus", Value: 1},
+		{Key: "recordStats", Value: 1},
+	}
+	res := client.Database("admin").RunCommand(ctx, query)
+	if res.Err() != nil {
+		return ss, errors.Wrap(res.Err(), "GetHostInfo.serverStatus")
 	}
-	defer session.Close()
-	session.SetMode(mgo.Monotonic, true)
 
-	if err := session.DB("admin").Run(bson.D{{"serverStatus", 1}, {"recordStats", 1}}, &ss); err != nil {
-		return ss, errors.Wrap(err, "GetHostInfo.serverStatus")
+	if err := res.Decode(&ss); err != nil {
+		return ss, errors.Wrap(err, "cannot decode serverStatus/recordStats")
 	}
 
 	return ss, nil
 }
 
-func GetQueryField(doc proto.SystemProfile) (bson.M, error) {
+func GetQueryField(doc proto.SystemProfile) (primitive.M, error) {
 	// Proper way to detect if protocol used is "op_msg" or "op_command"
 	// would be to look at "doc.Protocol" field,
 	// however MongoDB 3.0 doesn't have that field
@@ -248,7 +262,7 @@ func GetQueryField(doc proto.SystemProfile) (bson.M, error) {
 		if doc.Op == "update" || doc.Op == "remove" {
 			if squery, ok := query.Map()["q"]; ok {
 				// just an extra check to ensure this type assertion won't fail
-				if ssquery, ok := squery.(bson.M); ok {
+				if ssquery, ok := squery.(primitive.M); ok {
 					return ssquery, nil
 				}
 				return nil, CANNOT_GET_QUERY_ERROR
@@ -284,7 +298,7 @@ func GetQueryField(doc proto.SystemProfile) (bson.M, error) {
 	//
 	if squery, ok := query.Map()["query"]; ok {
 		// just an extra check to ensure this type assertion won't fail
-		if ssquery, ok := squery.(bson.M); ok {
+		if ssquery, ok := squery.(primitive.M); ok {
 			return ssquery, nil
 		}
 		return nil, CANNOT_GET_QUERY_ERROR
@@ -292,16 +306,24 @@ func GetQueryField(doc proto.SystemProfile) (bson.M, error) {
 
 	// "query" in MongoDB 3.2+ is better structured and always has a "filter" subkey:
 	if squery, ok := query.Map()["filter"]; ok {
-		if ssquery, ok := squery.(bson.M); ok {
+		if ssquery, ok := squery.(primitive.M); ok {
 			return ssquery, nil
 		}
 		return nil, CANNOT_GET_QUERY_ERROR
 	}
 
 	// {"ns":"test.system.js","op":"query","query":{"find":"system.js"}}
-	if len(query) == 1 && query[0].Name == "find" {
-		return bson.M{}, nil
+	if len(query) == 1 && query[0].Key == "find" {
+		return primitive.M{}, nil
 	}
 
 	return query.Map(), nil
 }
+
+// GetClientForHost returns a new *mongo.Client using a copy of the original connection options where
+// the host is being replaced by the newHost and the connection is set to be direct to the instance.
+func GetClientForHost(co *options.ClientOptions, newHost string) (*mongo.Client, error) {
+	newOptions := options.MergeClientOptions(co, &options.ClientOptions{Hosts: []string{newHost}})
+	newOptions.SetDirect(true)
+	return mongo.NewClient(newOptions)
+}
