@@ -2,6 +2,7 @@ package main
 
 import (
 	"bytes"
+	"context"
 	"encoding/json"
 	"fmt"
 	"html/template"
@@ -19,29 +20,31 @@ import (
 	"github.com/percona/percona-toolkit/src/go/mongolib/util"
 	"github.com/percona/percona-toolkit/src/go/pt-mongodb-summary/oplog"
 	"github.com/percona/percona-toolkit/src/go/pt-mongodb-summary/templates"
-	"github.com/percona/pmgo"
 	"github.com/pkg/errors"
 	"github.com/shirou/gopsutil/process"
 	log "github.com/sirupsen/logrus"
-	"gopkg.in/mgo.v2"
-	"gopkg.in/mgo.v2/bson"
+	"go.mongodb.org/mongo-driver/bson/primitive"
+	"go.mongodb.org/mongo-driver/mongo"
+	"go.mongodb.org/mongo-driver/mongo/options"
 )
 
 const (
 	TOOLNAME = "pt-mongodb-summary"
 
-	DEFAULT_AUTHDB             = "admin"
-	DEFAULT_HOST               = "localhost:27017"
-	DEFAULT_LOGLEVEL           = "warn"
-	DEFAULT_RUNNINGOPSINTERVAL = 1000 // milliseconds
-	DEFAULT_RUNNINGOPSSAMPLES  = 5
-	DEFAULT_OUTPUT_FORMAT      = "text"
+	DefaultAuthDB             = "admin"
+	DefaultHost               = "mongodb://localhost:27017"
+	DefaultLogLevel           = "warn"
+	DefaultRunningOpsInterval = 1000 // milliseconds
+	DefaultRunningOpsSamples  = 5
+	DefaultOutputFormat       = "text"
+	typeMongos                = "mongos"
 )
 
 var (
-	Build     string = "01-01-1980"
-	GoVersion string = "1.8"
-	Version   string = "3.0.1"
+	Build     string = "01-01-1980" // nolint
+	GoVersion string = "1.8"        // nolint
+	Version   string = "3.0.1"      // nolint
+	Commit    string                // nolint
 )
 
 type TimedStats struct {
@@ -88,8 +91,8 @@ type procInfo struct {
 }
 
 type security struct {
-	Users       int
-	Roles       int
+	Users       int64
+	Roles       int64
 	Auth        string
 	SSL         string
 	BindIP      string
@@ -99,14 +102,14 @@ type security struct {
 
 type databases struct {
 	Databases []struct {
-		Name       string           `bson:"name"`
-		SizeOnDisk int64            `bson:"sizeOnDisk"`
-		Empty      bool             `bson:"empty"`
-		Shards     map[string]int64 `bson:"shards"`
+		Name string `bson:"name"`
+		// SizeOnDisk int64            `bson:"sizeOnDisk"`
+		// Empty      bool             `bson:"empty"`
+		// Shards     map[string]int64 `bson:"shards"`
 	} `bson:"databases"`
-	TotalSize   int64 `bson:"totalSize"`
-	TotalSizeMb int64 `bson:"totalSizeMb"`
-	OK          bool  `bson:"ok"`
+	TotalSize   int64   `bson:"totalSize"`
+	TotalSizeMb int64   `bson:"totalSizeMb"`
+	OK          float64 `bson:"ok"`
 }
 
 type clusterwideInfo struct {
@@ -123,21 +126,21 @@ type clusterwideInfo struct {
 	Chunks                  []proto.ChunksByCollection
 }
 
-type options struct {
-	Help               bool
+type cliOptions struct {
 	Host               string
 	User               string
 	Password           string
 	AuthDB             string
 	LogLevel           string
+	OutputFormat       string
+	SSLCAFile          string
+	SSLPEMKeyFile      string
+	RunningOpsSamples  int
+	RunningOpsInterval int
+	Help               bool
 	Version            bool
 	NoVersionCheck     bool
 	NoRunningOps       bool
-	OutputFormat       string
-	RunningOpsSamples  int
-	RunningOpsInterval int
-	SSLCAFile          string
-	SSLPEMKeyFile      string
 }
 
 type collectedInfo struct {
@@ -152,7 +155,6 @@ type collectedInfo struct {
 }
 
 func main() {
-
 	opts, err := parseFlags()
 	if err != nil {
 		log.Errorf("cannot get parameters: %s", err.Error())
@@ -178,6 +180,7 @@ func main() {
 		fmt.Println(TOOLNAME)
 		fmt.Printf("Version %s\n", Version)
 		fmt.Printf("Build: %s using %s\n", Build, GoVersion)
+		fmt.Printf("Commit: %s\n", Commit)
 		return
 	}
 
@@ -186,76 +189,63 @@ func main() {
 		advice, err := versioncheck.CheckUpdates(TOOLNAME, Version)
 		if err != nil {
 			log.Infof("cannot check version updates: %s", err.Error())
-		} else {
-			if advice != "" {
-				log.Infof(advice)
-			}
+		} else if advice != "" {
+			log.Infof(advice)
 		}
 	}
 
-	di := &pmgo.DialInfo{
-		Username:      opts.User,
-		Password:      opts.Password,
-		Addrs:         []string{opts.Host},
-		FailFast:      true,
-		Source:        opts.AuthDB,
-		SSLCAFile:     opts.SSLCAFile,
-		SSLPEMKeyFile: opts.SSLPEMKeyFile,
-	}
-
-	log.Debugf("Connecting to the db using:\n%+v", di)
-	dialer := pmgo.NewDialer()
-
-	hostnames, err := util.GetHostnames(dialer, di)
-	log.Debugf("hostnames: %v", hostnames)
-
-	session, err := dialer.DialWithInfo(di)
+	ctx := context.Background()
+	clientOptions := getClientOptions(opts)
+	client, err := mongo.NewClient(clientOptions)
 	if err != nil {
-		message := fmt.Sprintf("Cannot connect to %q", di.Addrs[0])
-		if di.Username != "" || di.Password != "" {
-			message += fmt.Sprintf(" using user: %q", di.Username)
-			if strings.HasPrefix(di.Password, "=") {
-				message += " (probably you are using = with -p or -u instead of a blank space)"
-			}
-		}
-		message += fmt.Sprintf(". %s", err.Error())
-		log.Errorf(message)
-		os.Exit(1)
+		log.Fatalf("Cannot get a MongoDB client: %s", err)
 	}
-	defer session.Close()
-	session.SetMode(mgo.Monotonic, true)
+	if err := client.Connect(ctx); err != nil {
+		log.Fatalf("Cannot connect to MongoDB: %s", err)
+	}
+	defer client.Disconnect(ctx) // nolint
+
+	hostnames, err := util.GetHostnames(ctx, client)
+	if err != nil {
+		log.Errorf("Cannot get hostnames: %s", err)
+	}
+	log.Debugf("hostnames: %v", hostnames)
 
 	ci := &collectedInfo{}
 
-	ci.HostInfo, err = GetHostinfo(session)
+	ci.HostInfo, err = getHostInfo(ctx, client)
 	if err != nil {
-		message := fmt.Sprintf("Cannot get host info for %q: %s", di.Addrs[0], err.Error())
+		message := fmt.Sprintf("Cannot get host info for %q: %s", opts.Host, err.Error())
 		log.Errorf(message)
 		os.Exit(2)
 	}
 
-	if ci.ReplicaMembers, err = util.GetReplicasetMembers(dialer, di); err != nil {
+	if ci.ReplicaMembers, err = util.GetReplicasetMembers(ctx, clientOptions); err != nil {
 		log.Warnf("[Error] cannot get replicaset members: %v\n", err)
 		os.Exit(2)
 	}
 	log.Debugf("replicaMembers:\n%+v\n", ci.ReplicaMembers)
 
 	if opts.RunningOpsSamples > 0 && opts.RunningOpsInterval > 0 {
-		if ci.RunningOps, err = GetOpCountersStats(session, opts.RunningOpsSamples, time.Duration(opts.RunningOpsInterval)*time.Millisecond); err != nil {
+		ci.RunningOps, err = getOpCountersStats(
+			ctx, client, opts.RunningOpsSamples,
+			time.Duration(opts.RunningOpsInterval)*time.Millisecond,
+		)
+		if err != nil {
 			log.Printf("[Error] cannot get Opcounters stats: %v\n", err)
 		}
 	}
 
 	if ci.HostInfo != nil {
-		if ci.SecuritySettings, err = GetSecuritySettings(session, ci.HostInfo.Version); err != nil {
+		if ci.SecuritySettings, err = getSecuritySettings(ctx, client, ci.HostInfo.Version); err != nil {
 			log.Errorf("[Error] cannot get security settings: %v\n", err)
 		}
 	} else {
 		log.Warn("Cannot check security settings since host info is not available (permissions?)")
 	}
 
-	if ci.OplogInfo, err = oplog.GetOplogInfo(hostnames, di); err != nil {
-		log.Info("Cannot get Oplog info: %v\n", err)
+	if ci.OplogInfo, err = oplog.GetOplogInfo(ctx, hostnames, clientOptions); err != nil {
+		log.Infof("Cannot get Oplog info: %s\n", err)
 	} else {
 		if len(ci.OplogInfo) == 0 {
 			log.Info("oplog info is empty. Skipping")
@@ -265,14 +255,14 @@ func main() {
 	}
 
 	// individual servers won't know about this info
-	if ci.HostInfo.NodeType == "mongos" {
-		if ci.ClusterWideInfo, err = GetClusterwideInfo(session); err != nil {
+	if ci.HostInfo.NodeType == typeMongos {
+		if ci.ClusterWideInfo, err = getClusterwideInfo(ctx, client); err != nil {
 			log.Printf("[Error] cannot get cluster wide info: %v\n", err)
 		}
 	}
 
-	if ci.HostInfo.NodeType == "mongos" {
-		if ci.BalancerStats, err = GetBalancerStats(session); err != nil {
+	if ci.HostInfo.NodeType == typeMongos {
+		if ci.BalancerStats, err = GetBalancerStats(ctx, client); err != nil {
 			log.Printf("[Error] cannot get balancer stats: %v\n", err)
 		}
 	}
@@ -300,48 +290,63 @@ func formatResults(ci *collectedInfo, format string) ([]byte, error) {
 		buf = new(bytes.Buffer)
 
 		t := template.Must(template.New("replicas").Parse(templates.Replicas))
-		t.Execute(buf, ci.ReplicaMembers)
+		if err := t.Execute(buf, ci.ReplicaMembers); err != nil {
+			return nil, errors.Wrap(err, "cannot parse replicas section of the output template")
+		}
 
 		t = template.Must(template.New("hosttemplateData").Parse(templates.HostInfo))
-		t.Execute(buf, ci.HostInfo)
+		if err := t.Execute(buf, ci.HostInfo); err != nil {
+			return nil, errors.Wrap(err, "cannot parse hosttemplateData section of the output template")
+		}
 
 		t = template.Must(template.New("runningOps").Parse(templates.RunningOps))
-		t.Execute(buf, ci.RunningOps)
+		if err := t.Execute(buf, ci.RunningOps); err != nil {
+			return nil, errors.Wrap(err, "cannot parse runningOps section of the output template")
+		}
 
 		t = template.Must(template.New("ssl").Parse(templates.Security))
-		t.Execute(buf, ci.SecuritySettings)
+		if err := t.Execute(buf, ci.SecuritySettings); err != nil {
+			return nil, errors.Wrap(err, "cannot parse ssl section of the output template")
+		}
 
 		if ci.OplogInfo != nil && len(ci.OplogInfo) > 0 {
 			t = template.Must(template.New("oplogInfo").Parse(templates.Oplog))
-			t.Execute(buf, ci.OplogInfo[0])
+			if err := t.Execute(buf, ci.OplogInfo[0]); err != nil {
+				return nil, errors.Wrap(err, "cannot parse oplogInfo section of the output template")
+			}
 		}
 
 		t = template.Must(template.New("clusterwide").Parse(templates.Clusterwide))
-		t.Execute(buf, ci.ClusterWideInfo)
+		if err := t.Execute(buf, ci.ClusterWideInfo); err != nil {
+			return nil, errors.Wrap(err, "cannot parse clusterwide section of the output template")
+		}
 
 		t = template.Must(template.New("balancer").Parse(templates.BalancerStats))
-		t.Execute(buf, ci.BalancerStats)
+		if err := t.Execute(buf, ci.BalancerStats); err != nil {
+			return nil, errors.Wrap(err, "cannot parse balancer section of the output template")
+		}
 	}
 
 	return buf.Bytes(), nil
 }
 
-func GetHostinfo(session pmgo.SessionManager) (*hostInfo, error) {
-
+func getHostInfo(ctx context.Context, client *mongo.Client) (*hostInfo, error) {
 	hi := proto.HostInfo{}
-	if err := session.Run(bson.M{"hostInfo": 1}, &hi); err != nil {
-		log.Debugf("run('hostInfo') error: %s", err.Error())
+	if err := client.Database("admin").RunCommand(ctx, primitive.M{"hostInfo": 1}).Decode(&hi); err != nil {
+		log.Debugf("run('hostInfo') error: %s", err)
 		return nil, errors.Wrap(err, "GetHostInfo.hostInfo")
 	}
 
 	cmdOpts := proto.CommandLineOptions{}
-	err := session.DB("admin").Run(bson.D{{"getCmdLineOpts", 1}, {"recordStats", 1}}, &cmdOpts)
+	query := primitive.D{{Key: "getCmdLineOpts", Value: 1}, {Key: "recordStats", Value: 1}}
+	err := client.Database("admin").RunCommand(ctx, query).Decode(&cmdOpts)
 	if err != nil {
 		return nil, errors.Wrap(err, "cannot get command line options")
 	}
 
 	ss := proto.ServerStatus{}
-	if err := session.DB("admin").Run(bson.D{{"serverStatus", 1}, {"recordStats", 1}}, &ss); err != nil {
+	query = primitive.D{{Key: "serverStatus", Value: 1}, {Key: "recordStats", Value: 1}}
+	if err := client.Database("admin").RunCommand(ctx, query).Decode(&ss); err != nil {
 		return nil, errors.Wrap(err, "GetHostInfo.serverStatus")
 	}
 
@@ -350,14 +355,14 @@ func GetHostinfo(session pmgo.SessionManager) (*hostInfo, error) {
 		pi.Error = err
 	}
 
-	nodeType, _ := getNodeType(session)
+	nodeType, _ := getNodeType(ctx, client)
 	procCount, _ := countMongodProcesses()
 
 	i := &hostInfo{
 		Hostname:          hi.System.Hostname,
 		HostOsType:        hi.Os.Type,
 		HostSystemCPUArch: hi.System.CpuArch,
-		DBPath:            "", // Sets default. It will be overriden later if necessary
+		DBPath:            "", // Sets default. It will be overridden later if necessary
 
 		ProcessName:      ss.Process,
 		ProcProcessCount: procCount,
@@ -390,19 +395,19 @@ func countMongodProcesses() (int, error) {
 		if err != nil {
 			continue
 		}
-		if name, _ := p.Name(); name == "mongod" || name == "mongos" {
+		if name, _ := p.Name(); name == "mongod" || name == typeMongos {
 			count++
 		}
 	}
 	return count, nil
 }
 
-func GetClusterwideInfo(session pmgo.SessionManager) (*clusterwideInfo, error) {
+func getClusterwideInfo(ctx context.Context, client *mongo.Client) (*clusterwideInfo, error) {
 	var databases databases
 
-	err := session.Run(bson.M{"listDatabases": 1}, &databases)
+	err := client.Database("admin").RunCommand(ctx, primitive.M{"listDatabases": 1}).Decode(&databases)
 	if err != nil {
-		return nil, errors.Wrap(err, "GetClusterwideInfo.listDatabases ")
+		return nil, errors.Wrap(err, "getClusterwideInfo.listDatabases ")
 	}
 
 	cwi := &clusterwideInfo{
@@ -410,19 +415,24 @@ func GetClusterwideInfo(session pmgo.SessionManager) (*clusterwideInfo, error) {
 	}
 
 	for _, db := range databases.Databases {
-		collections, err := session.DB(db.Name).CollectionNames()
+		cursor, err := client.Database(db.Name).ListCollections(ctx, primitive.M{})
 		if err != nil {
 			continue
 		}
 
-		cwi.TotalCollectionsCount += len(collections)
-		for _, collName := range collections {
-			var collStats proto.CollStats
-			err := session.DB(db.Name).Run(bson.M{"collStats": collName}, &collStats)
-			if err != nil {
-				continue
+		for cursor.Next(ctx) {
+			c := proto.CollectionEntry{}
+			if err := cursor.Decode(&c); err != nil {
+				return nil, errors.Wrap(err, "cannot decode ListCollections doc")
 			}
 
+			var collStats proto.CollStats
+			err := client.Database(db.Name).RunCommand(ctx, primitive.M{"collStats": c.Name}).Decode(&collStats)
+			if err != nil {
+				return nil, errors.Wrapf(err, "cannot get info for collection %s.%s", db.Name, c.Name)
+			}
+			cwi.TotalCollectionsCount++
+
 			if collStats.Sharded {
 				cwi.ShardedDataSize += collStats.Size
 				cwi.ShardedColsCount++
@@ -432,14 +442,16 @@ func GetClusterwideInfo(session pmgo.SessionManager) (*clusterwideInfo, error) {
 			cwi.UnshardedDataSize += collStats.Size
 			cwi.UnshardedColsCount++
 		}
-
 	}
 
 	cwi.UnshardedColsCount = cwi.TotalCollectionsCount - cwi.ShardedColsCount
 	cwi.ShardedDataSizeScaled, cwi.ShardedDataSizeScale = sizeAndUnit(cwi.ShardedDataSize)
 	cwi.UnshardedDataSizeScaled, cwi.UnshardedDataSizeScale = sizeAndUnit(cwi.UnshardedDataSize)
 
-	cwi.Chunks, _ = getChunksCount(session)
+	cwi.Chunks, err = getChunksCount(ctx, client)
+	if err != nil {
+		return nil, errors.Wrap(err, "cannot get chunks information")
+	}
 
 	return cwi, nil
 }
@@ -456,7 +468,7 @@ func sizeAndUnit(size int64) (float64, string) {
 	return newSize, unit[idx]
 }
 
-func GetSecuritySettings(session pmgo.SessionManager, ver string) (*security, error) {
+func getSecuritySettings(ctx context.Context, client *mongo.Client, ver string) (*security, error) {
 	s := security{
 		Auth: "disabled",
 		SSL:  "disabled",
@@ -470,7 +482,7 @@ func GetSecuritySettings(session pmgo.SessionManager, ver string) (*security, er
 	}
 
 	cmdOpts := proto.CommandLineOptions{}
-	err = session.DB("admin").Run(bson.D{{"getCmdLineOpts", 1}, {"recordStats", 1}}, &cmdOpts)
+	err = client.Database("admin").RunCommand(ctx, primitive.D{{"getCmdLineOpts", 1}, {"recordStats", 1}}).Decode(&cmdOpts)
 	if err != nil {
 		return nil, errors.Wrap(err, "cannot get command line options")
 	}
@@ -498,29 +510,29 @@ func GetSecuritySettings(session pmgo.SessionManager, ver string) (*security, er
 			isPrivate, err := isPrivateNetwork(strings.TrimSpace(ip))
 			if !isPrivate && err == nil {
 				if s.Auth == "enabled" {
-					s.WarningMsgs = append(s.WarningMsgs, fmt.Sprintf("Warning: You might be insecure (bind ip %s is public)", ip))
+					s.WarningMsgs = append(
+						s.WarningMsgs,
+						fmt.Sprintf("Warning: You might be insecure (bind ip %s is public)", ip),
+					)
 				} else {
-					s.WarningMsgs = append(s.WarningMsgs, fmt.Sprintf("Error. You are insecure: bind ip %s is public and auth is disabled", ip))
+					s.WarningMsgs = append(
+						s.WarningMsgs,
+						fmt.Sprintf("Error. You are insecure: bind ip %s is public and auth is disabled", ip),
+					)
 				}
 			} else {
 				if ip != "127.0.0.1" && ip != extIP {
-					s.WarningMsgs = append(s.WarningMsgs, fmt.Sprintf("WARNING: You might be insecure. IP binding %s is not localhost", ip))
+					s.WarningMsgs = append(
+						s.WarningMsgs,
+						fmt.Sprintf("WARNING: You might be insecure. IP binding %s is not localhost", ip),
+					)
 				}
 			}
 		}
 	}
 
-	// On some servers, like a mongos with config servers, this fails if session mode is Monotonic
-	// On some other servers like a secondary in a replica set, this fails if the session mode is Strong.
-	// Lets try both
-	newSession := session.Clone()
-	defer newSession.Close()
-
-	newSession.SetMode(mgo.Strong, true)
-
-	if s.Users, s.Roles, err = getUserRolesCount(newSession); err != nil {
-		newSession.SetMode(mgo.Monotonic, true)
-		if s.Users, s.Roles, err = getUserRolesCount(newSession); err != nil {
+	if s.Users, s.Roles, err = getUserRolesCount(ctx, client); err != nil {
+		if s.Users, s.Roles, err = getUserRolesCount(ctx, client); err != nil {
 			return nil, errors.Wrap(err, "cannot get security settings.")
 		}
 	}
@@ -528,23 +540,22 @@ func GetSecuritySettings(session pmgo.SessionManager, ver string) (*security, er
 	return &s, nil
 }
 
-func getUserRolesCount(session pmgo.SessionManager) (int, int, error) {
-	users, err := session.DB("admin").C("system.users").Count()
+func getUserRolesCount(ctx context.Context, client *mongo.Client) (int64, int64, error) {
+	users, err := client.Database("admin").Collection("system.users").CountDocuments(ctx, primitive.M{})
 	if err != nil {
 		return 0, 0, errors.Wrap(err, "cannot get users count")
 	}
 
-	roles, err := session.DB("admin").C("system.roles").Count()
+	roles, err := client.Database("admin").Collection("system.roles").CountDocuments(ctx, primitive.M{})
 	if err != nil {
 		return 0, 0, errors.Wrap(err, "cannot get roles count")
 	}
 	return users, roles, nil
 }
 
-func getNodeType(session pmgo.SessionManager) (string, error) {
+func getNodeType(ctx context.Context, client *mongo.Client) (string, error) {
 	md := proto.MasterDoc{}
-	err := session.Run("isMaster", &md)
-	if err != nil {
+	if err := client.Database("admin").RunCommand(ctx, primitive.M{"isMaster": 1}).Decode(&md); err != nil {
 		return "", err
 	}
 
@@ -553,12 +564,13 @@ func getNodeType(session pmgo.SessionManager) (string, error) {
 	} else if md.Msg == "isdbgrid" {
 		// isdbgrid is always the msg value when calling isMaster on a mongos
 		// see http://docs.mongodb.org/manual/core/sharded-cluster-query-router/
-		return "mongos", nil
+		return typeMongos, nil
 	}
 	return "mongod", nil
 }
 
-func GetOpCountersStats(session pmgo.SessionManager, count int, sleep time.Duration) (*opCounters, error) {
+func getOpCountersStats(ctx context.Context, client *mongo.Client, count int,
+	sleep time.Duration) (*opCounters, error) {
 	oc := &opCounters{}
 	prevOpCount := &opCounters{}
 	ss := proto.ServerStatus{}
@@ -570,7 +582,7 @@ func GetOpCountersStats(session pmgo.SessionManager, count int, sleep time.Durat
 	// count + 1 because we need 1st reading to stablish a base to measure variation
 	for i := 0; i < count+1; i++ {
 		<-ticker.C
-		err := session.DB("admin").Run(bson.D{{"serverStatus", 1}, {"recordStats", 1}}, &ss)
+		err := client.Database("admin").RunCommand(ctx, primitive.D{{"serverStatus", 1}, {"recordStats", 1}}).Decode(&ss)
 		if err != nil {
 			return nil, err
 		}
@@ -694,7 +706,7 @@ func getProcInfo(pid int32, templateData *procInfo) error {
 	//proc, err := process.NewProcess(templateData.ServerStatus.Pid)
 	proc, err := process.NewProcess(pid)
 	if err != nil {
-		return fmt.Errorf("cannot get process %d\n", pid)
+		return fmt.Errorf("cannot get process %d", pid)
 	}
 	ct, err := proc.CreateTime()
 	if err != nil {
@@ -714,38 +726,8 @@ func getProcInfo(pid int32, templateData *procInfo) error {
 	return nil
 }
 
-func getDbsAndCollectionsCount(hostnames []string) (int, int, error) {
-	dbnames := make(map[string]bool)
-	colnames := make(map[string]bool)
-
-	for _, hostname := range hostnames {
-		session, err := mgo.Dial(hostname)
-		if err != nil {
-			continue
-		}
-		dbs, err := session.DatabaseNames()
-		if err != nil {
-			continue
-		}
-
-		for _, dbname := range dbs {
-			dbnames[dbname] = true
-			cols, err := session.DB(dbname).CollectionNames()
-			if err != nil {
-				continue
-			}
-			for _, colname := range cols {
-				colnames[dbname+"."+colname] = true
-			}
-		}
-	}
-
-	return len(dbnames), len(colnames), nil
-}
-
-func GetBalancerStats(session pmgo.SessionManager) (*proto.BalancerStats, error) {
-
-	scs, err := GetShardingChangelogStatus(session)
+func GetBalancerStats(ctx context.Context, client *mongo.Client) (*proto.BalancerStats, error) {
+	scs, err := GetShardingChangelogStatus(ctx, client)
 	if err != nil {
 		return nil, err
 	}
@@ -773,16 +755,25 @@ func GetBalancerStats(session pmgo.SessionManager) (*proto.BalancerStats, error)
 	return s, nil
 }
 
-func GetShardingChangelogStatus(session pmgo.SessionManager) (*proto.ShardingChangelogStats, error) {
+func GetShardingChangelogStatus(ctx context.Context, client *mongo.Client) (*proto.ShardingChangelogStats, error) {
 	var qresults []proto.ShardingChangelogSummary
-	coll := session.DB("config").C("changelog")
-	match := bson.M{"time": bson.M{"$gt": time.Now().Add(-240 * time.Hour)}}
-	group := bson.M{"_id": bson.M{"event": "$what", "note": "$details.note"}, "count": bson.M{"$sum": 1}}
+	coll := client.Database("config").Collection("changelog")
+	match := primitive.M{"time": primitive.M{"$gt": time.Now().Add(-240 * time.Hour)}}
+	group := primitive.M{"_id": primitive.M{"event": "$what", "note": "$details.note"}, "count": primitive.M{"$sum": 1}}
 
-	err := coll.Pipe([]bson.M{{"$match": match}, {"$group": group}}).All(&qresults)
+	cursor, err := coll.Aggregate(ctx, []primitive.M{{"$match": match}, {"$group": group}})
 	if err != nil {
 		return nil, errors.Wrap(err, "GetShardingChangelogStatus.changelog.find")
 	}
+	defer cursor.Close(ctx)
+
+	for cursor.Next(ctx) {
+		res := proto.ShardingChangelogSummary{}
+		if err := cursor.Decode(&res); err != nil {
+			return nil, errors.Wrap(err, "cannot decode GetShardingChangelogStatus")
+		}
+		qresults = append(qresults, res)
+	}
 
 	return &proto.ShardingChangelogStats{
 		Items: &qresults,
@@ -848,14 +839,14 @@ func externalIP() (string, error) {
 	return "", errors.New("are you connected to the network?")
 }
 
-func parseFlags() (*options, error) {
-	opts := &options{
-		Host:               DEFAULT_HOST,
-		LogLevel:           DEFAULT_LOGLEVEL,
-		RunningOpsSamples:  DEFAULT_RUNNINGOPSSAMPLES,
-		RunningOpsInterval: DEFAULT_RUNNINGOPSINTERVAL, // milliseconds
-		AuthDB:             DEFAULT_AUTHDB,
-		OutputFormat:       DEFAULT_OUTPUT_FORMAT,
+func parseFlags() (*cliOptions, error) {
+	opts := &cliOptions{
+		Host:               DefaultHost,
+		LogLevel:           DefaultLogLevel,
+		RunningOpsSamples:  DefaultRunningOpsSamples,
+		RunningOpsInterval: DefaultRunningOpsInterval, // milliseconds
+		AuthDB:             DefaultAuthDB,
+		OutputFormat:       DefaultOutputFormat,
 	}
 
 	gop := getopt.New()
@@ -864,17 +855,22 @@ func parseFlags() (*options, error) {
 	gop.BoolVarLong(&opts.NoVersionCheck, "no-version-check", 'c', "", "Default: Don't check for updates")
 
 	gop.StringVarLong(&opts.User, "username", 'u', "", "Username to use for optional MongoDB authentication")
-	gop.StringVarLong(&opts.Password, "password", 'p', "", "Password to use for optional MongoDB authentication").SetOptional()
+	gop.StringVarLong(&opts.Password, "password", 'p', "", "Password to use for optional MongoDB authentication").
+		SetOptional()
 	gop.StringVarLong(&opts.AuthDB, "authenticationDatabase", 'a', "admin",
 		"Database to use for optional MongoDB authentication. Default: admin")
-	gop.StringVarLong(&opts.LogLevel, "log-level", 'l', "error", "Log level: panic, fatal, error, warn, info, debug. Default: error")
+	gop.StringVarLong(&opts.LogLevel, "log-level", 'l', "error",
+		"Log level: panic, fatal, error, warn, info, debug. Default: error")
 	gop.StringVarLong(&opts.OutputFormat, "output-format", 'f', "text", "Output format: text, json. Default: text")
 
 	gop.IntVarLong(&opts.RunningOpsSamples, "running-ops-samples", 's',
-		fmt.Sprintf("Number of samples to collect for running ops. Default: %d", opts.RunningOpsSamples))
+		fmt.Sprintf("Number of samples to collect for running ops. Default: %d", opts.RunningOpsSamples),
+	)
 
 	gop.IntVarLong(&opts.RunningOpsInterval, "running-ops-interval", 'i',
-		fmt.Sprintf("Interval to wait betwwen running ops samples in milliseconds. Default %d milliseconds", opts.RunningOpsInterval))
+		fmt.Sprintf("Interval to wait betwwen running ops samples in milliseconds. Default %d milliseconds",
+			opts.RunningOpsInterval),
+	)
 
 	gop.StringVarLong(&opts.SSLCAFile, "sslCAFile", 0, "SSL CA cert file used for authentication")
 	gop.StringVarLong(&opts.SSLPEMKeyFile, "sslPEMKeyFile", 0, "SSL client PEM file used for authentication")
@@ -886,6 +882,7 @@ func parseFlags() (*options, error) {
 		opts.Host = gop.Arg(0)
 		gop.Parse(gop.Args())
 	}
+
 	if gop.IsSet("password") && opts.Password == "" {
 		print("Password: ")
 		pass, err := gopass.GetPasswd()
@@ -894,6 +891,9 @@ func parseFlags() (*options, error) {
 		}
 		opts.Password = string(pass)
 	}
+	if !strings.HasPrefix(opts.Host, "mongodb://") {
+		opts.Host = "mongodb://" + opts.Host
+	}
 	if opts.Help {
 		gop.PrintUsage(os.Stdout)
 		return nil, nil
@@ -905,16 +905,38 @@ func parseFlags() (*options, error) {
 	return opts, nil
 }
 
-func getChunksCount(session pmgo.SessionManager) ([]proto.ChunksByCollection, error) {
+func getChunksCount(ctx context.Context, client *mongo.Client) ([]proto.ChunksByCollection, error) {
 	var result []proto.ChunksByCollection
 
-	c := session.DB("config").C("chunks")
-	query := bson.M{"$group": bson.M{"_id": "$ns", "count": bson.M{"$sum": 1}}}
+	c := client.Database("config").Collection("chunks")
+	query := primitive.M{"$group": primitive.M{"_id": "$ns", "count": primitive.M{"$sum": 1}}}
 
 	// db.getSiblingDB('config').chunks.aggregate({$group:{_id:"$ns",count:{$sum:1}}})
-	err := c.Pipe([]bson.M{query}).All(&result)
+	cursor, err := c.Aggregate(ctx, []primitive.M{query})
 	if err != nil {
 		return nil, err
 	}
+	for cursor.Next(ctx) {
+		res := proto.ChunksByCollection{}
+		if err := cursor.Decode(&res); err != nil {
+			return nil, errors.Wrap(err, "cannot decode chunks aggregation")
+		}
+		result = append(result, res)
+	}
 	return result, nil
 }
+
+func getClientOptions(opts *cliOptions) *options.ClientOptions {
+	clientOptions := options.Client().ApplyURI(opts.Host)
+	credential := options.Credential{}
+	if opts.User != "" {
+		credential.Username = opts.User
+		clientOptions.SetAuth(credential)
+	}
+	if opts.Password != "" {
+		credential.Password = opts.Password
+		credential.PasswordSet = true
+		clientOptions.SetAuth(credential)
+	}
+	return clientOptions
+}
